#@+leo-ver=4
#@+node:@file ../scripts/foldersize_leo.py
"""
Calculate foldersizes; generate XML file for leo to display results.

Parameters:
   foldersize_leo.py <xmlfile> [directory] *
   
Returns for each directory:
	- the number of directories.
	- the number of files.
	- the total size.
	
Only tested on Windows.

Does not take into account compression on Windows.

Ignores non-ascii filenames.
"""

#@@language python

import getopt,os,stat,string,sys
import leo_interface

comma = ','
intsize = 20

#@+others
#@+node:normalize
def normalize(filename):
	
	"""This should probably be changed for Linux"""
	return string.replace(os.path.normpath(string.lower(filename)), '\\', '/')

#@-node:normalize
#@+node:filesize
def filesize(fullname):
	return long(os.stat(fullname) [stat.ST_SIZE])
#@-node:filesize
#@+node:format
def format(value, size):

	"""Insert 'comma' into the representation of value for easier reading"""
	if value==0:
		result = '0'
	else:
		result = ''
		negative = value < 0
		if negative:
		  value = - value
		underbar = 4
		while value != 0:
			underbar = underbar - 1
			if underbar == 0:
				result = comma + result
				underbar = 3
			newvalue = value / 10
			rest     = int(value - newvalue*10)
			result = `rest` + result
			value = newvalue
		if negative:
			result = '-' + result
	if len(result) < size:
		result = ' ' * (size - len(result)) + result
	return result
#@-node:format
#@+node:class TreeItem
class TreeItem(leo_interface.leo_node):
	
	"""Create a tree node for each directory.
	
	Put the information about the files in the directory
	into the body text of the tree node."""

	#@	@+others
	#@+node:__init__
	def __init__(self, path):
		leo_interface.leo_node.__init__(self)
		self.path = path
		self.filecount, self.dircount, self.filesize = 0, 1, 0
		filenames, subdirs = self.GetSubList()
		if subdirs:
			for subdir in subdirs:
				self.filecount += subdir.filecount
				self.dircount  += subdir.dircount
				self.filesize  += subdir.filesize
				self.add_child(subdir)
		body_lines = []
		self.filecount += len(filenames)
		for filename in filenames:
			try:
				size = filesize(filename)
			except EnvironmentError, value:
				print "Sorry:, could not stat %s" % self.path, value
				size = 0
			self.filesize += size
			body_lines.append((size, filename))
		body_lines.sort()
		body_lines.reverse()
		body_lines = [self.format_file(name, size) for size, name in body_lines]
		self.set_headline(self.format_directory(
			name      = os.path.basename(path),
			dircount  = self.dircount,
			filecount = self.filecount,
			filesize  = self.filesize))
		self.set_body(string.join(body_lines, "\n"))
	#@-node:__init__
	#@+node:format_directory
	def format_directory(self, name, dircount, filecount, filesize):
	
		name = os.path.basename(name)
		return ("%s %s %s %s" % (name,
								format(filesize, intsize),
								format(filecount, intsize),
								format(dircount, intsize),
								))
	
	#@-node:format_directory
	#@+node:format_file
	def format_file(self, name, filesize):
	
		name = os.path.basename(name)
		return "%s %s" % (format(filesize, intsize), name)
	#@-node:format_file
	#@+node:GetSubList
	def GetSubList(self):
		"""
		Look into the directory.
	
		Visit all subdirectories, and create
		nodes for them.
	
		Create strings for all files in the current directory.
	
		Returns a list of files, and a list of node representing
		the subdirectories."""
		files, directory_nodes = [], []
		try:
			names = os.listdir(self.path)
		except os.error:
			return files, directory_nodes
		join = os.path.join
		isdir = os.path.isdir
		for name in names:
			skip = False
			for ch in name:
				if ord(ch) > 127:
					print "Skipping ", name
					# Such names seem to be able to cause trouble
					# during reading...
					skip = True
			if skip:
				continue
			normpath = normalize(join(self.path, name))
			if isdir(normpath):
				directory_nodes.append(TreeItem(normpath))
			else:
				files.append(normpath)
		directory_nodes.sort(lambda a, b: cmp(b.filesize, a.filesize))
		return files, directory_nodes
	#@-node:GetSubList
	#@-others
#@nonl
#@-node:class TreeItem
#@+node:test
def test():

	try:
		opts, values = getopt.getopt(sys.argv[1:], '', [])
	except:
		print __doc__
		return

	if len(values) < 1:
		print __doc__
		return

	resultfile = values[0]
	leotree  = leo_interface.leotree()
	roots = [TreeItem(root) for root in values[1:]]
	for root in roots:
		leotree.add_child(root)
	file = open(resultfile, "w")
	leotree.gen(file)
	file.close()
#@nonl
#@-node:test
#@-others

if __name__ == '__main__':
   test()
#@nonl
#@-node:@file ../scripts/foldersize_leo.py
#@-leo
